@@ -14,7 +14,7 @@ import sys
 import argparse
 import os
 
-import pygit2
+import thingy.git2 as git
 
 ################################################################################
 
@@ -26,61 +26,27 @@ def main():
     parser = argparse.ArgumentParser(description='Report top-level directory of the current git working tree.')
     parser.add_argument('--parent', '-p', action='store_true',
                         help='If we are already at the top of the working tree, check if the parent directory is in a working tree and output the top-level directory of that tree.')
-    parser.add_argument('--repo', '-r', action='store_true',
-                        help='If we are already at the top of the working tree, look for a parent directory with a repo control file (.repo directory or .mrconfig file) and output that directory.')
-    parser.add_argument('--dir', '-d', action='store', default=os.getcwd(),
+    parser.add_argument('--dir', '-d', action='store', default=None,
                         help='Find the location of the top-level directory in the working tree starting at the specified directory')
-    parser.add_argument('level', nargs='?', type=int, default=0, help='Number of levels below the top-level directory to report')
+
     args = parser.parse_args()
 
-    if not os.path.isdir(args.dir):
-        print(f'Unable to locate directory {args.dir}')
+    try:
+        start_dir = os.path.abspath(args.dir or os.getcwd())
+    except FileNotFoundError:
+        sys.stderr.write('Unable to determine initial directory\n')
         sys.exit(1)
 
-    # Try to get the current working tree
+    # Search for a .git directory in the current or parent directories
 
-    try:
-        git_dir = pygit2.Repository(args.dir).workdir
-    except pygit2.GitError:
-        print(f'Directory {args.dir} is not in a Git working tree')
-        sys.exit(2)
-
-    working_tree = os.path.abspath(os.path.join(git_dir, os.pardir))
+    working_tree = git.working_tree(start_dir)
 
     # If we are in a working tree and also looking for the parent working
     # tree, check if we are at the top of the current tree, and, if so,
     # hop up a level and try again.
 
-    if args.parent and working_tree:
-        current_directory = os.getcwd()
-
-        if os.path.samefile(working_tree, current_directory):
-            os.chdir('..')
-
-            working_tree = pygit2.Repository(current_directory).workdir
-
-    # If we are also looking for a multi-repo control file or directory, and haven't
-    # found the git working tree root scan up the tree until we find one.
-
-    if args.repo and not working_tree:
-        while True:
-            working_tree = os.getcwd()
-
-            if os.path.isdir('.repo') or os.path.isfile('../.mrconfig'):
-                break
-
-            if working_tree == '/':
-                sys.exit(2)
-
-            os.chdir('..')
-
-    # Output the result, if we have one
-
-    if args.level:
-        start = args.dir.split('/')
-        working = working_tree.split('/')
-
-        working_tree = os.path.join(working_tree, '/'.join(start[len(working):len(working) + int(args.level)]))
+    if args.parent and working_tree == start_dir:
+        working_tree = git.working_tree(os.path.join(working_tree, os.pardir))
 
     if working_tree:
         print(working_tree)
