@@ -15,7 +15,8 @@
     * Functions will raise exceptions on error. If the underlying git command
       returns an error, a git.GitError() exception is raised.
 
-    * TODO: Cache list of branches when git.branches/isbranch called
+    * TODO: [ ] Cache list of branches when git.branches/isbranch called
+    * TODO: [ ] API change - git_run_status should raise an exception on failure and just return the git output
 """
 ################################################################################
 
@@ -37,6 +38,11 @@ import thingy.gitlab as gitlab
 
 (LOCAL, GLOBAL, SYSTEM) = list(range(3))
 
+# Options always passed to Git (disable autocorrect to stop it running the wrong command
+# if an invalid command name has been specified).
+
+STANDARD_GIT_OPTIONS = ['-c', 'help.autoCorrect=never']
+
 ################################################################################
 
 class GitError(run.RunError):
@@ -53,56 +59,91 @@ def git(cmd, stdout=None, stderr=None, path=None):
         to get the exception.
         Optionally redirect stdout and stderr as specified. """
 
-    logging.debug('Running git %s', ' '.join(cmd))
-
-    git_cmd = ['git']
+    git_cmd = ['git'] + STANDARD_GIT_OPTIONS
 
     if path:
         git_cmd += ['-C', path]
 
+    git_cmd += cmd if isinstance(cmd, list) else [cmd]
+
+    logging.debug('Running %s', ' '.join(git_cmd + cmd))
+
+    sys.stdout.flush()
+    sys.stderr.flush()
+
     try:
-        return run.run(git_cmd + cmd, stdout=stdout, stderr=stderr)
+        return run.run(git_cmd, stdout=stdout, stderr=stderr)
     except run.RunError as exc:
         raise GitError(exc.msg, exc.status)
 
 ################################################################################
 
-def git_run_status(cmd, stdout=None, stderr=None, path=None):
+def git_run_status(cmd, stdout=None, stderr=None, path=None, redirect=True):
     """ Wrapper for run.run that returns the output and status, and
         does not raise an exception on error.
         Optionally redirect stdout and stderr as specified. """
 
     logging.debug('Running git %s', ' '.join(cmd))
 
-    git_cmd = ['git']
+    git_cmd = ['git'] + STANDARD_GIT_OPTIONS
 
     if path:
         git_cmd += ['-C', path]
 
-    result = subprocess.run(git_cmd + cmd,
-                            stdout=stdout or subprocess.PIPE,
-                            stderr=stderr or subprocess.PIPE,
-                            text=True, check=False,
-                            errors='ignore',
-                            universal_newlines=True)
+    git_cmd += cmd if isinstance(cmd, list) else [cmd]
+
+    sys.stdout.flush()
+    sys.stderr.flush()
+
+    if redirect:
+        result = subprocess.run(git_cmd,
+                                stdout=stdout or subprocess.PIPE,
+                                stderr=stderr or subprocess.PIPE,
+                                text=True, check=False,
+                                errors='ignore',
+                                universal_newlines=True)
+    else:
+        result = subprocess.run(git_cmd)
 
     return (result.stdout or result.stderr), result.returncode
 
 ################################################################################
 
-def clone(reponame, directory=None):
+def git_dir(path=None):
+    """ Return the relative path to the .git directory """
+
+    if not path:
+        path = os.getcwd()
+
+    return pygit2.discover_repository(path)
+
+################################################################################
+
+def working_tree(path=None):
+    """ Location of the current working tree or None if we are not in a working tree """
+
+    repo_dir = git_dir(path=path)
+
+    if repo_dir:
+        return os.path.abspath(os.path.join(repo_dir, os.pardir))
+
+    return None
+
+################################################################################
+
+def clone(reponame, path=None):
     """ Clone a repo """
 
     cmd = ['clone', reponame]
 
-    if directory:
-        cmd.append(directory)
+    if path:
+        cmd.append(path)
 
     return git(cmd)
 
 ################################################################################
 
-def init(reponame, bare=False):
+def init(reponame, bare=False, path=None):
     """ Initialise a new working tree """
 
     cmd = ['init']
@@ -112,11 +153,11 @@ def init(reponame, bare=False):
 
     cmd.append(reponame)
 
-    return git(cmd)
+    return git(cmd, path=path)
 
 ################################################################################
 
-def iscommit(commit, remote=False, remote_only=False):
+def iscommit(commit, remote=False, remote_only=False, path=None):
     """ Return True if "commit" is a valid SHA1, branch or tag
         If remote==True then if there are no direct matches it will also
         check for a matching remote branch
@@ -127,7 +168,7 @@ def iscommit(commit, remote=False, remote_only=False):
     if not remote_only:
         cmd = ['cat-file', '-t', commit]
         try:
-            result = git(cmd)[0]
+            result = git(cmd, path=path)[0]
 
             return result in ('commit', 'tag')
         except GitError:
@@ -136,7 +177,7 @@ def iscommit(commit, remote=False, remote_only=False):
     # Optionally look for matching remote branch
 
     if remote or remote_only:
-        for branch in branches(all=True):
+        for branch in branches(all=True, path=path):
             if branch.startswith('remotes/'):
                 localbranch = '/'.join(branch.split('/')[2:])
 
@@ -157,24 +198,44 @@ def branch(branchname='HEAD', path=None):
 
 ################################################################################
 
-def tag():
+def tag(path=None):
     """ If the current commit is tagged, return the tag(s) or None """
 
     try:
-        return git(['describe', '--tags', '--exact-match'])[0]
+        return git(['describe', '--tags', '--exact-match'], path=path)[0]
     except GitError:
         return None
 
 ################################################################################
 
-def tags():
+def tags(path=None):
     """ Return the list of tags in the current repo """
 
-    return git(['tag'])
+    return git(['tag', '--list'], path=path)
 
 ################################################################################
 
-def current_commit(short=False):
+def tag_delete(tag, push=False, path=None):
+    """Delete a tag, optionally pushing the deletion"""
+
+    git(['tag', '-d', tag], path=path)
+
+    if push:
+        git(['push', 'origin', '--delete', tag], path=path)
+
+################################################################################
+
+def tag_apply(tag, push=False, path=None):
+    """Apply a tag, optionally pushing it"""
+
+    git(['tag', tag], path=path)
+
+    if push:
+        git(['push', 'origin', tag], path=path)
+
+################################################################################
+
+def current_commit(short=False, path=None):
     """ Return the SHA1 of the current commit """
 
     cmd = ['rev-parse']
@@ -184,7 +245,7 @@ def current_commit(short=False):
 
     cmd.append('HEAD')
 
-    return git(cmd)[0]
+    return git(cmd, path=path)[0]
 
 ################################################################################
 
@@ -217,23 +278,23 @@ def checkout(branch, create=False, path=None):
 
 ################################################################################
 
-def merge(branch):
+def merge(branch, path=None):
     """ Merge a branch """
 
     cmd = ['merge', branch]
 
-    return git(cmd)
+    return git(cmd, path=path)
 
 ################################################################################
 
-def abort_merge():
+def abort_merge(path=None):
     """ Abort the current merge """
 
-    return git(['merge', '--abort'])
+    return git(['merge', '--abort'], path=path)
 
 ################################################################################
 
-def set_upstream(branch, upstream=None):
+def set_upstream(branch, upstream=None, path=None):
     """ Set the default upstream branch """
 
     if not upstream:
@@ -241,7 +302,7 @@ def set_upstream(branch, upstream=None):
 
     cmd = ['branch', f'--set-upstream-to={upstream}', branch]
 
-    return git(cmd)
+    return git(cmd, path=path)
 
 ################################################################################
 
@@ -257,7 +318,7 @@ def fetch(all=False, path=None):
 
 ################################################################################
 
-def rebase_required(branch, parent):
+def rebase_required(branch, parent, path=None):
     """ Return True if the specified branch needs to be rebased against its
         parent.
     """
@@ -265,8 +326,8 @@ def rebase_required(branch, parent):
     # Find the latest commit on the parent branch and the most recent commit
     # that both branches have in common.
 
-    parent_tip = git(['show-ref', '--heads', '-s', parent])
-    common_commit = git(['merge-base', parent, branch])
+    parent_tip = git(['show-ref', '--heads', '-s', parent], path=path)
+    common_commit = git(['merge-base', parent, branch], path=path)
 
     # Different commits, so rebase is required
 
@@ -274,52 +335,52 @@ def rebase_required(branch, parent):
 
 ################################################################################
 
-def rebase(branch):
+def rebase(branch, path=None):
     """ Rebase the current branch against the specified branch """
 
-    return git_run_status(['rebase', branch])
+    return git_run_status(['rebase', branch], path=path)
 
 ################################################################################
 
-def abort_rebase():
+def abort_rebase(path=None):
     """ Abort the current rebase """
 
-    return git(['rebase', '--abort'])
+    return git(['rebase', '--abort'], path=path)
 
 ################################################################################
 
-def rebasing():
+def rebasing(path=None):
     """ Return True if currently rebasing, False otherwise """
 
-    gitdir = git_dir()
+    gitdir = git_dir(path=path)
 
     return os.path.isdir(os.path.join(gitdir, 'rebase-apply')) or \
         os.path.isdir(os.path.join(gitdir, 'rebase-merge'))
 
 ################################################################################
 
-def bisecting():
+def bisecting(path=None):
     """ Return True if currently rebasing, False otherwise """
 
-    gitdir = git_dir()
+    gitdir = git_dir(path=path)
 
     return os.path.isfile(os.path.join(gitdir, 'BISECT_START'))
 
 ################################################################################
 
-def merging():
+def merging(path=None):
     """ Return True if currently merging, False otherwise """
 
-    gitdir = git_dir()
+    gitdir = git_dir(path=path)
 
     return os.path.isfile(os.path.join(gitdir, 'MERGE_MODE'))
 
 ################################################################################
 
-def remotes():
+def remotes(path=None):
     """ Return the list of git remotes """
 
-    repo = pygit2.Repository(os.getcwd())
+    repo = pygit2.Repository(git_dir(path=path))
 
     git_remotes = {}
 
@@ -330,10 +391,10 @@ def remotes():
 
 ################################################################################
 
-def remote_names():
+def remote_names(path=None):
     """ Return the list of remote names """
 
-    repo = pygit2.Repository(os.getcwd())
+    repo = pygit2.Repository(git_dir(path=path))
 
     results = list(repo.remotes.names())
 
@@ -341,10 +402,10 @@ def remote_names():
 
 ################################################################################
 
-def project(short=False):
+def project(short=False, path=None):
     """ Return the name of the current git project """
 
-    git_remotes = remotes()
+    git_remotes = remotes(path=path)
     name = ''
 
     for remote in git_remotes:
@@ -370,7 +431,7 @@ def project(short=False):
 
 ################################################################################
 
-def status_info(ignored=False, untracked=False):
+def status_info(ignored=False, untracked=False, path=None):
     """ Git status, optionally include files ignored in .gitignore and/or
         untracked files.
         Returns data in the same dictionary format as used by commit_info() """
@@ -383,7 +444,7 @@ def status_info(ignored=False, untracked=False):
     if untracked:
         cmd.append('--untracked-files=all')
 
-    results = git(cmd)
+    results = git(cmd, path=path)
 
     # Dictionary of results, indexed by filename where the status is 2 characters
     # the first representing the state of the file in the index and the second the state
@@ -437,6 +498,9 @@ def status(ignored=False, untracked=False, path=None):
         stats = []
         stats.append(result[0:2])
 
+        if not untracked and result[0] == '?':
+            continue
+
         name = result[3:]
         if ' -> ' in name:
             stats += name.split(' -> ', 1)
@@ -449,39 +513,17 @@ def status(ignored=False, untracked=False, path=None):
 
 ################################################################################
 
-def working_tree(start_dir=None):
-    """ Location of the current working tree or None if we are not in a working tree """
-
-    repo_dir = git_dir(start_dir)
-
-    if repo_dir:
-        return os.path.abspath(os.path.join(repo_dir, '..'))
-
-    return None
-
-################################################################################
-
-def git_dir(start_dir=None):
-    """ Return the relative path to the .git directory """
-
-    if not start_dir:
-        start_dir = os.getcwd()
-
-    return pygit2.discover_repository(start_dir)
-
-################################################################################
-
-def tree_path(filename):
+def tree_path(filename, path=None):
     """ Normalise a filename (absolute or relative to the current directory)
         so that it is relative to the top-level directory of the working tree """
 
-    git_tree = working_tree()
+    git_tree = working_tree(path=path)
 
     return os.path.relpath(filename, git_tree)
 
 ################################################################################
 
-def difftool(commit_1=None, commit_2=None, files=None, tool=None):
+def difftool(commit_1=None, commit_2=None, files=None, tool=None, path=None):
     """ Run git difftool """
 
     cmd = ['difftool']
@@ -503,11 +545,11 @@ def difftool(commit_1=None, commit_2=None, files=None, tool=None):
         else:
             cmd += files
 
-    return git(cmd)
+    return git(cmd, path=path)
 
 ################################################################################
 
-def commit_info(commit_1=None, commit_2=None, paths=None):
+def commit_info(commit_1=None, commit_2=None, paths=None, path=None):
     """ Return details of changes either in single commit (defaulting to the most
         recent one) or between two commits, optionally restricted a path or paths
     """
@@ -530,7 +572,7 @@ def commit_info(commit_1=None, commit_2=None, paths=None):
         else:
             params += paths
 
-    results = git(params)
+    results = git(params, path=path)
 
     # Parse the output
 
@@ -555,7 +597,7 @@ def commit_info(commit_1=None, commit_2=None, paths=None):
 
 ################################################################################
 
-def diff(commit=None, renames=True, copies=True, relative=False):
+def diff(commit=None, renames=True, copies=True, relative=False, path=None):
     """ Return a list of differences between two commits, working tree and a commit or working tree and head """
 
     if commit:
@@ -581,17 +623,17 @@ def diff(commit=None, renames=True, copies=True, relative=False):
 
     cmd += commit
 
-    return git(cmd)
+    return git(cmd, path=path)
 
 ################################################################################
 
-def diff_status(commit1, commit2='HEAD'):
+def diff_status(commit1, commit2='HEAD', path=None):
     """ Return True if there is no difference between the two commits, False otherwise """
 
     cmd = ['diff', '--no-patch', '--exit-code', commit1, commit2]
 
     try:
-        git(cmd)
+        git(cmd, path=path)
     except GitError:
         return False
 
@@ -599,30 +641,31 @@ def diff_status(commit1, commit2='HEAD'):
 
 ################################################################################
 
-def show(revision, filename, outfile=None):
+def show(revision, filename, outfile=None, path=None):
     """ Return the output from git show revision:filename """
 
-    return git(['show', f'{revision}:{filename}'], stdout=outfile)
+    return git(['show', f'{revision}:{filename}'], stdout=outfile, path=path)
 
 ################################################################################
 
-def add(files):
+def add(files, path=None):
     """ Add file to git """
 
-    return git(['add'] + files)
+    return git(['add'] + files, path=path)
 
 ################################################################################
 
-def rm(files):
+def rm(files, path=None):
     """ Remove files from git """
 
-    return git(['rm'] + files)
+    return git(['rm'] + files, path=path)
 
 ################################################################################
 
 def commit(files=None,
            message=None,
-           all=False, amend=False, foreground=False, patch=False, dry_run=False):
+           all=False, amend=False, foreground=False, patch=False, dry_run=False,
+           path=None):
     """ Commit files to git """
 
     cmd = ['commit']
@@ -647,16 +690,17 @@ def commit(files=None,
         cmd += ['-m', message]
 
     if foreground:
-        return git(cmd, stdout=sys.stdout, stderr=sys.stderr)
+        return git(cmd, stdout=sys.stdout, stderr=sys.stderr, path=path)
 
-    return git(cmd)
+    return git(cmd, path=path)
 
 ################################################################################
 
 def push(all=False, mirror=False, tags=False, atomic=False, dry_run=False,
          follow_tags=False, receive_pack=False, repo=None, force=False, delete=False,
          prune=False, verbose=False, set_upstream=False, push_options=[], signed=None,
-         force_with_lease=False, no_verify=False, repository=None, refspec=None):
+         force_with_lease=False, no_verify=False, repository=None, refspec=None,
+         path=None):
     """ Push commits to a remote """
 
     cmd = ['push']
@@ -720,18 +764,18 @@ def push(all=False, mirror=False, tags=False, atomic=False, dry_run=False,
         for ref in refspec:
             cmd.append(ref)
 
-    return git(cmd)
+    return git(cmd, path=path)
 
 ################################################################################
 
-def reset(sha1):
+def reset(sha1, path=None):
     """ Run git reset """
 
-    return git(['reset', sha1])
+    return git(['reset', sha1], path=path)
 
 ################################################################################
 
-def config_get(section, key, source=LOCAL, defaultvalue=None):
+def config_get(section, key, source=LOCAL, defaultvalue=None, path=None):
     """ Return the specified configuration entry
         Returns a default value if no matching configuration entry exists """
 
@@ -745,13 +789,13 @@ def config_get(section, key, source=LOCAL, defaultvalue=None):
     cmd += ['--get', f'{section}.{key}']
 
     try:
-        return git(cmd)[0]
+        return git(cmd, path=path)[0]
     except GitError:
         return defaultvalue
 
 ################################################################################
 
-def config_set(section, key, value, source=LOCAL):
+def config_set(section, key, value, source=LOCAL, path=None):
     """ Set a configuration entry """
 
     cmd = ['config']
@@ -763,11 +807,11 @@ def config_set(section, key, value, source=LOCAL):
 
     cmd += ['--replace-all', f'{section}.{key}', value]
 
-    return git(cmd)
+    return git(cmd, path=path)
 
 ################################################################################
 
-def config_rm(section, key, source=LOCAL):
+def config_rm(section, key, source=LOCAL, path=None):
     """ Remove a configuration entry """
 
     cmd = ['config']
@@ -779,11 +823,11 @@ def config_rm(section, key, source=LOCAL):
 
     cmd += ['--unset', f'{section}.{key}']
 
-    return git(cmd)
+    return git(cmd, path=path)
 
 ################################################################################
 
-def ref(fields=('objectname'), sort=None, remotes=False):
+def ref(fields=('objectname'), sort=None, remotes=False, path=None):
     """ Wrapper for git for-each-ref """
 
     cmd = ['for-each-ref']
@@ -800,12 +844,12 @@ def ref(fields=('objectname'), sort=None, remotes=False):
     if remotes:
         cmd.append('refs/remotes/origin')
 
-    for output in git(cmd):
+    for output in git(cmd, path=path):
         yield output.split('\0')
 
 ################################################################################
 
-def branches(all=False):
+def branches(all=False, path=None):
     """ Return a list of all the branches in the current repo """
 
     cmd = ['branch']
@@ -814,7 +858,7 @@ def branches(all=False):
         cmd.append('--all')
 
     results = []
-    for output in git(cmd):
+    for output in git(cmd, path=path):
         if ' -> ' not in output and '(HEAD detached at ' not in output:
             results.append(output[2:])
 
@@ -822,7 +866,7 @@ def branches(all=False):
 
 ################################################################################
 
-def delete_branch(branch, force=False, remote=False):
+def delete_branch(branch, force=False, remote=False, path=None):
     """ Delete a branch, optionally forcefully and/or including the
         remote tracking branch """
 
@@ -836,11 +880,11 @@ def delete_branch(branch, force=False, remote=False):
 
     cmd.append(branch)
 
-    return git(cmd)
+    return git(cmd, path=path)
 
 ################################################################################
 
-def remote_prune(remote, dry_run=False):
+def remote_prune(remote, dry_run=False, path=None):
     """ Return a list of remote tracking branches that no longer exist on the
         specified remote """
 
@@ -849,7 +893,7 @@ def remote_prune(remote, dry_run=False):
     if dry_run:
         cmd.append('--dry-run')
 
-    results = git(cmd)
+    results = git(cmd, path=path)
 
     prunable_branches = []
 
@@ -864,36 +908,43 @@ def remote_prune(remote, dry_run=False):
 
 ################################################################################
 
-def get_commits(commit1, commit2):
+def get_commits(commit1, commit2, path=None):
     """ Get a list of commits separating two commits """
 
-    return git(['rev-list', commit1, f'^{commit2}'])
+    return git(['rev-list', commit1, f'^{commit2}'], path=path)
 
 ################################################################################
 
-def commit_count(commit1, commit2):
+def commit_count(commit1, commit2, path=None):
     """ Get a count of the number of commits between two commits """
 
-    return int(git(['rev-list', '--count', commit1, '^%s' % commit2])[0])
+    return int(git(['rev-list', '--count', commit1, f'^{commit2}'], path=path)[0])
 
 ################################################################################
 
-def branch_name(branch):
+def branch_name(branch, path=None):
     """ Return the full name of a branch given an abbreviation - e.g. @{upstream}
         for the upstream branch """
 
-    return git(['rev-parse', '--abbrev-ref', '--symbolic-full-name', branch])[0]
+    return git(['rev-parse', '--abbrev-ref', '--symbolic-full-name', branch], path=path)[0]
 
 ################################################################################
 
-def author(commit):
+def author(commit, path=None):
     """ Return the author of a commit """
 
-    return git(['show', '--format=format:%an', commit])[0]
+    return git(['show', '--format=format:%an', commit], path=path)[0]
 
 ################################################################################
 
-def files(dir=None):
+def commit_changes(commit='HEAD', path=None):
+    """Return a list of the files changed in a commit"""
+
+    return git(['show', '--name-only', '--pretty=format:', commit], path=path)
+
+################################################################################
+
+def files(dir=None, path=None):
     """ Return the output from 'git ls-files' """
 
     cmd = ['ls-files']
@@ -901,20 +952,20 @@ def files(dir=None):
     if dir:
         cmd.append(dir)
 
-    return git(cmd)
+    return git(cmd, path=path)
 
 ################################################################################
 
-def stash():
+def stash(path=None):
     """ Return the list of stashed items (if any) """
 
     cmd = ['stash', 'list']
 
-    return git(cmd)
+    return git(cmd, path=path)
 
 ################################################################################
 
-def parents(commit=None, ignore=None):
+def parents(commit=None, ignore=None, path=None):
     """ Look at the commits down the history of the specified branch,
         looking for another branch or branches that also contain the same commit.
         The first found is the parent (or equally-likely parents) of the
@@ -926,16 +977,16 @@ def parents(commit=None, ignore=None):
 
     # Get the history of the branch
 
-    current_branch = commit or branch('HEAD')
+    current_branch = commit or branch('HEAD', path=path)
 
-    current_history = git(['rev-list', current_branch])
+    current_history = git(['rev-list', current_branch], path=path)
 
     # Look down the commits on the current branch for other branches that have
     # the same commit, using the ignore pattern if there is one.
 
     for distance, ancestor in enumerate(current_history):
         branches = []
-        for brnch in git(['branch', '--contains', ancestor]):
+        for brnch in git(['branch', '--contains', ancestor], path=path):
             brnch = brnch[2:]
             if brnch != current_branch and '(HEAD detached at' not in brnch:
                 if not ignore or (ignore and not fnmatch.fnmatch(brnch, ignore)):
@@ -950,13 +1001,11 @@ def parents(commit=None, ignore=None):
 
 ################################################################################
 
-def find_common_ancestor(branch1='HEAD', branch2='master'):
+def find_common_ancestor(branch1='HEAD', branch2='master', path=None):
     """ Find the first (oldest) commit that the two branches have in common
         i.e. the point where one branch was forked from the other """
 
-    common = git(['merge-base', branch1, branch2])[0]
-
-    return common
+    return git(['merge-base', branch1, branch2], path=path)[0]
 
 ################################################################################
 
@@ -1007,7 +1056,7 @@ _GREP_NON_BOOL_OPTLIST = \
         ('parent_basename', '--parent-basename')
     )
 
-def grep(pattern, git_dir=None, work_tree=None, options=None, wildcards=None):
+def grep(pattern, git_dir=None, work_tree=None, options=None, wildcards=None, path=None):
     """ Run git grep - takes a painfully large number of options passed
         as a dictionary. """
 
@@ -1044,23 +1093,23 @@ def grep(pattern, git_dir=None, work_tree=None, options=None, wildcards=None):
         cmd.append('--')
         cmd += wildcards
 
-    return git_run_status(cmd)
+    return git_run_status(cmd, path=path)
 
 ################################################################################
 
-def isbranch(branchname):
+def isbranch(branchname, path=None):
     """ Return true if the specified branch exists """
 
-    return branchname in branches(True)
+    return branchname in branches(True, path=path)
 
 ################################################################################
 
-def default_branch():
+def default_branch(path=None):
     """ Return the name of the default branch, attempting to interrogate GitLab
         if the repo appears to have been cloned from there and falling back to
         returning whichever one of 'develop', 'main' or 'master' exists. """
 
-    remote_list = remotes()
+    remote_list = remotes(path=path)
     if remote_list:
         for name in remote_list:
             if 'gitlab' in remote_list[name]:
@@ -1068,7 +1117,7 @@ def default_branch():
                 repo = remote_list[name].split(':')[1]
 
                 if not url.startswith('http://') or not url.startswith('https://'):
-                    url = 'https://%s' % url
+                    url = f'https://{url}'
 
                 if repo.endswith('.git'):
                     repo = repo[:-4]
@@ -1080,7 +1129,7 @@ def default_branch():
                 except gitlab.GitLabError:
                     return None
 
-    git_branches = branches()
+    git_branches = branches(path=path)
 
     for branch in ('develop', 'main', 'master'):
         if branch in git_branches:
@@ -1090,11 +1139,7 @@ def default_branch():
 
 ################################################################################
 
-    return None
-
-################################################################################
-
-def matching_branch(branchname, case=False):
+def matching_branch(branchname, case=False, path=None):
     """ Look for a branch matching the specified name and return it
         out if it is an exact match or there is only one partial
         match. If there are multiple branches that match, return them
@@ -1106,7 +1151,7 @@ def matching_branch(branchname, case=False):
         otherwise, it just checks for a branches containing the branchname
         as a substring. """
 
-    all_branches = branches(all=True)
+    all_branches = branches(all=True, path=path)
 
     # Always return exact matches
 
@@ -1158,7 +1203,7 @@ def matching_branch(branchname, case=False):
 
 ################################################################################
 
-def update(clean=False, all=False):
+def update(clean=False, all=False, path=None):
     """ Run git update (which is a thingy command, and may end up as a module
         but for the moment, we'll treat it as any other git command) """
 
@@ -1170,18 +1215,18 @@ def update(clean=False, all=False):
     if all:
         cmd.append('--all')
 
-    return git(cmd)
+    return git(cmd, path=path)
 
 ################################################################################
 
-def object_type(name):
+def object_type(name, path=None):
     """ Return the git object type (commit, tag, blob, ...) """
 
-    return git(['cat-file', '-t', name])[0]
+    return git(['cat-file', '-t', name], path=path)[0]
 
 ################################################################################
 
-def matching_commit(name):
+def matching_commit(name, path=None):
     """ Similar to matching_branch() (see above).
         If the name uniquely matches a branch, return that
         If it matches multiple branches return a list
@@ -1197,7 +1242,7 @@ def matching_commit(name):
 
     # Look for at least one matching branch
 
-    matches = matching_branch(name)
+    matches = matching_branch(name, path=path)
 
     if matches:
         return matches
@@ -1215,7 +1260,7 @@ def matching_commit(name):
     # Look for a matching commit
 
     try:
-        commit_type = object_type(name)
+        commit_type = object_type(name, path=path)
 
         if commit_type == 'commit':
             matches = [name]
@@ -1226,7 +1271,7 @@ def matching_commit(name):
 
 ################################################################################
 
-def log(branch1, branch2=None):
+def log(branch1, branch2=None, path=None):
     """ Return the git log between the given commits """
 
     if branch2:
@@ -1234,7 +1279,38 @@ def log(branch1, branch2=None):
     else:
         cmd = ['log', '-n1', branch1]
 
-    return git(cmd)
+    return git(cmd, path=path)
+
+################################################################################
+
+def clean(recurse=False, force=False, dry_run=False, quiet=False,
+          exclude=None, ignore_rules=False, remove_only_ignored=False, path=None):
+    """ Run git clean """
+
+    cmd = ['clean']
+
+    if recurse:
+        cmd.append('-d')
+
+    if force:
+        cmd.append('--force')
+
+    if dry_run:
+        cmd.append('--dry-run')
+
+    if quiet:
+        cmd.append('--quiet')
+
+    if exclude:
+        cmd += ['--exclude', exclude]
+
+    if ignore_rules:
+        cmd.append('-x')
+
+    if remove_only_ignored:
+        cmd.append('-X')
+
+    return git(cmd, path=path)
 
 ################################################################################
 # Entry point
@@ -1276,7 +1352,7 @@ if __name__ == '__main__':
 
         print('')
 
-        with open('newfile.txt', 'w') as newfile:
+        with open('newfile.txt', 'w', encoding='utf8') as newfile:
             newfile.write('THIS IS A TEST')
 
         print('Adding and committing "newfile.txt"')
@@ -1298,7 +1374,7 @@ if __name__ == '__main__':
         print('Commit info for HEAD  %s' % commit_info('HEAD'))
 
     except GitError as exc:
-        sys.stderr.write('ERROR: %s' % exc.msg)
+        sys.stderr.write(f'ERROR: {exc.msg}')
         sys.exit(1)
 
     finally:
